﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Configuration;

namespace DataSyncClient
{
    enum DownloadType { Inserts, Updates }

    /**
     * Downloads new or updated constituent records from Convio to a local XML file.
     */
    class ConstituentDownloader
    {

        private Convio.SoapClient client;
        private Convio.Session session;

        public ConstituentDownloader(Convio.SoapClient client, Convio.Session session)
        {
            this.client = client;
            this.session = session;
        }

        public void download(DownloadType type)
        {
            String typeLabel = type.ToString().ToLower();
            String partitionId = ConfigurationSettings.AppSettings["partitionId"];

            /**
             * Create an in-memory data structure to hold the downloaded records.
             */
            DataSet downloadConstituentsData = new DataSet(type.ToString());
            DataTable downloadConstituentsTable = new DataTable("Constituent");
            downloadConstituentsData.Tables.Add(downloadConstituentsTable);
            downloadConstituentsTable.Columns.Add(new DataColumn("ConsId"));
            downloadConstituentsTable.Columns.Add(new DataColumn("PrimaryEmail"));
            downloadConstituentsTable.Columns.Add(new DataColumn("FirstName"));
            downloadConstituentsTable.Columns.Add(new DataColumn("LastName"));
            downloadConstituentsTable.Columns.Add(new DataColumn("MemberId"));
            downloadConstituentsTable.Columns.Add(new DataColumn("UserName"));
            downloadConstituentsTable.Columns.Add(new DataColumn("OriginSrcCode"));
            downloadConstituentsTable.Columns.Add(new DataColumn("OriginSubSrcCode"));

            /**
             * Specify the set of fields to return in the result.
             */
            String[] fields = { "ConsId", "ConsName", "UserName", "MemberId", 
                                       "OriginSrcCode", "OriginSubSrcCode", "PrimaryEmail" };

            /**
             * Request the records one page at a time.  Continue asking for the next
             * page until the response is empty, indicating that the entire result
             * has been covered.
             */
            int page = 1;
            while (true)
            {
                Convio.Record[] records = (type == DownloadType.Inserts) ?
                    client.GetIncrementalInserts(session, partitionId, "Constituent", page.ToString(), "100", fields) :
                    client.GetIncrementalUpdates(session, partitionId, "Constituent", page.ToString(), "100", fields);
                if (records == null || records.Length == 0)
                {
                    if (page == 1)
                    {
                        System.Console.WriteLine("No constituent " + typeLabel + " to download");
                    }
                    break;
                }
                int start = (page - 1) * 100 + 1;
                System.Console.WriteLine("Downloading constituent " + typeLabel + " " +
                    start + " to " + (start + records.Length));

                /**
                 * Copy each constituent from this page into the master list.
                 */
                foreach (Convio.Record record in records)
                {
                    Convio.Constituent constituent = (Convio.Constituent)record;
                    DataRow row = downloadConstituentsTable.Rows.Add();

                    row["ConsId"] = constituent.ConsId;
                    if (constituent.ConsName != null)
                    {
                        row["FirstName"] = constituent.ConsName.FirstName;
                        row["LastName"] = constituent.ConsName.LastName;
                    }
                    row["UserName"] = constituent.UserName;
                    row["MemberId"] = constituent.MemberId;
                    row["OriginSrcCode"] = constituent.OriginSrcCode;
                    row["OriginSubSrcCode"] = constituent.OriginSubsrcCode;
                    row["PrimaryEmail"] = constituent.PrimaryEmail;
                }

                page++;
            }

            /**
             * Write the accumulated master list to a file
             */
            String downloadConstituentsFile = ConfigurationSettings.AppSettings["downloadConstituent" + typeLabel + "File"];
            downloadConstituentsData.WriteXml(downloadConstituentsFile);
            downloadConstituentsData.Dispose();
        }
    }
}
