﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Configuration;

namespace DataSyncClient
{
    /**
     * Uploads new constituent records from a local XML file to Convio using the
     * Create operation.
     **/
    class ConstituentUploader
    {
        private Convio.SoapClient client;
        private Convio.Session session;

        public ConstituentUploader(Convio.SoapClient client, Convio.Session session)
        {
            this.client = client;
            this.session = session;
        }

        public void upload()
        {
            String partitionId = ConfigurationSettings.AppSettings["partitionId"];

            /**
             * Read new records (presumably dumped from a donor database or other offline system)
             * into an in-memory data table.
             **/
            String uploadConstituentsFile = ConfigurationSettings.AppSettings["uploadConstituentsDataFile"];
            System.Console.WriteLine("Uploading constituents from " + uploadConstituentsFile);

            DataSet uploadData = new DataSet();
            uploadData.ReadXml(uploadConstituentsFile);

            DataTable uploadTable = uploadData.Tables[0];

            /**
             * Iterate over the records and build a corresponding list of Constituent records for upload.
             **/
            List<Convio.Constituent> uploadList = new List<Convio.Constituent>();
            foreach (DataRow uploadRecord in uploadTable.Rows)
            {
                Convio.Constituent constituent = new Convio.Constituent();
                uploadList.Add(constituent);

                constituent.PrimaryEmail = uploadRecord["PrimaryEmail"].ToString();
                constituent.ConsName = new Convio.Name();
                constituent.ConsName.FirstName = uploadRecord["FirstName"].ToString();
                constituent.ConsName.LastName = uploadRecord["LastName"].ToString();
                constituent.MemberId = uploadRecord["MemberId"].ToString();
                constituent.UserName = uploadRecord["UserName"].ToString();
                constituent.OriginSrcCode = uploadRecord["OriginSrcCode"].ToString();
                constituent.OriginSubsrcCode = uploadRecord["OriginSubSrcCode"].ToString();

                String interestIds = uploadRecord["InterestId"].ToString();
                constituent.InterestId = interestIds.Split(',');

                String groupIds = uploadRecord["GroupId"].ToString();
                constituent.GroupId = groupIds.Split(',');
            }
            uploadData.Dispose();

            /**
             * Create an in-memory data structure for capturing the results for each uploaded record.
             **/
            DataSet uploadResultData = new DataSet("Results");
            DataTable uploadResultTable = new DataTable("Result");
            uploadResultData.Tables.Add(uploadResultTable);
            uploadResultTable.Columns.Add(new DataColumn("ResultCode"));
            uploadResultTable.Columns.Add(new DataColumn("Message"));
            uploadResultTable.Columns.Add(new DataColumn("ConsId"));
            uploadResultTable.Columns.Add(new DataColumn("MemberId"));
            uploadResultTable.Columns.Add(new DataColumn("PrimaryEmail"));

            int total = uploadList.Count;

            /**
             * Upload the records 50 at a time (the maximum allowed by the Create or Upload operation)
             **/
            for (int start = 0; start < total; start += 50)
            {
                int count = start + 50 < total ? 50 : total - start;
                System.Console.WriteLine("Uploading new constituents " + (start + 1) + " to " + (start + count));
                Convio.Constituent[] uploadRecords = uploadList.GetRange(start, count).ToArray();

                Convio.RecordModificationResult[] uploadRecordResults =
                    client.Create(session, partitionId, uploadRecords);

                foreach (Convio.RecordModificationResult uploadRecordResult in uploadRecordResults)
                {
                    DataRow row = uploadResultTable.Rows.Add();
                    row["ResultCode"] = uploadRecordResult.ResultCode.ToString();
                    row["Message"] = uploadRecordResult.Message.ToString();
                    Convio.Constituent constituentResult = (Convio.Constituent)uploadRecordResult.Record;
                    /**
                     * If the system was unable to successfully create or update a constituent, this
                     * part of the result may be null
                     **/
                    if (constituentResult != null)
                    {
                        row["ConsId"] = constituentResult.ConsId;
                        row["MemberId"] = constituentResult.MemberId;
                        row["PrimaryEmail"] = constituentResult.PrimaryEmail;
                    }
                }
            }

            /**
             * Dump the results to a text file for auditing
             **/
            String uploadResultsFile = ConfigurationSettings.AppSettings["uploadConstituentsResultFile"];
            uploadResultData.WriteXml(uploadResultsFile);
            uploadResultData.Dispose();
        }
    }
}
