﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;

using System.Data;
using System.Configuration;

namespace DataSyncClient
{
    /**
     * Sample command-line client for Convio Web Services
     * 
     * To run this sample, update the Convio Service Reference 
     * with the URL of the your client WSDL, and update the
     * log path, authentication credentials, and partition ID define
     * in app.config.
     **/
    class Program
    {
        static void Main(string[] args)
        {
            Convio.SoapClient client = new Convio.SoapClient();

            System.Console.WriteLine("Connecting to " + client.Endpoint.Address);

            Convio.Session session = new Convio.Session();

            /**
             * First request the Login operation and obtain a session token
             **/
            try
            {
                String userName = ConfigurationSettings.AppSettings["userName"];
                String password = ConfigurationSettings.AppSettings["password"];

                Convio.LoginResult result = client.Login(userName, password);
                System.Console.WriteLine("Login successful");

                session.SessionId = result.SessionId;

            }
            catch (FaultException<Convio.LoginFault> e)
            {
                System.Console.WriteLine("Login failed due to invalid credentials: " + e.Message);
                System.Console.WriteLine("Press return to exit");
                System.Console.ReadLine();

                return;
            }

            /**
             * Next attempt to start a synchronization session.  By default, the session
             * will cover changes since the start of the last successful session up to the
             * current time.
             * 
             * If the server returns a synchronization fault, it is usually because a previous
             * synchronization did not finish successfully.  Normally you will want to stop
             * and investigate, but for demo purposes we give you the option to simply end
             * the last session and continue.
             **/
            String partitionId = ConfigurationSettings.AppSettings["partitionId"];

            while (true)
            {
                try
                {
                    Convio.StartSynchronizationResult startResult =
                        client.StartSynchronization(session, partitionId, null, null, null);

                    System.Console.WriteLine("Synchronizing changes that occurred between " +
                        startResult.Start + " and " + startResult.End);

                    break;
                }
                catch (FaultException<Convio.SynchronizationFault> e)
                {
                    System.Console.WriteLine("Failed to start synchronization session: " + e.Message);
                    System.Console.WriteLine("Do you want to end the previous session and retry [enter yes or no]?");
                    String answer = System.Console.ReadLine();
                    if ("yes".Equals(answer.ToLower()))
                    {
                        client.EndSynchronization(session, partitionId);
                        continue;
                    }
                    return;
                }
            }

            /**
             * See the Uploader and Downloader helper classes for examples of reading and writing data from and to
             * data files.
             **/
            ConstituentUploader uploader = new ConstituentUploader(client, session);
            uploader.upload();

            ConstituentDownloader downloader = new ConstituentDownloader(client, session);
            downloader.download(DownloadType.Inserts);
            downloader.download(DownloadType.Updates);

            DonationDownloader donationDownloader = new DonationDownloader(client, session);
            donationDownloader.download(DownloadType.Inserts);

            client.EndSynchronization(session, partitionId);

            System.Console.WriteLine("Press return to exit");
            System.Console.ReadLine();
        }
    }
}
